module Stats exposing (..)

import BoundedInt exposing (BoundedInt)

type alias Stat =
    { name : String
    , dots : BoundedInt
    , statType : StatType
    }

type StatType
    = Sphere
    | Arete

new : List Stat
new =
    [ newSphere "Correspondence"
    , newSphere "Entropy"
    , newSphere "Forces"
    , newSphere "Life"
    , newSphere "Matter"
    , newSphere "Mind"
    , newSphere "Prime"
    , newSphere "Spirit"
    , newSphere "Time"
    , newArete
    ]

newSphere : String -> Stat
newSphere name =
    { name = name
    , dots = BoundedInt.new 0 0 5
    , statType = Sphere
    }

newArete : Stat
newArete =
    { name = "Arete"
    , dots = BoundedInt.new 1 0 10
    , statType = Arete
    }

spheresInList : List Stat -> List Stat
spheresInList stats =
    List.filter isSphere stats

isSphere : Stat -> Bool
isSphere stat =
    stat.statType == Sphere

areteInList : List Stat -> Stat
areteInList stats =
    let 
        maybeArete = List.head ( List.filter isArete stats )
    in
        case maybeArete of
            Just arete -> arete
            Nothing -> newArete

isArete : Stat -> Bool
isArete stat =
    stat.statType == Arete

dotsOfStatInList : List Stat -> String -> Maybe BoundedInt
dotsOfStatInList stats name =
    let
        maybeStat = stats
            |> List.filter ( isStat name )
            |> List.head
    in
        case maybeStat of
            Nothing -> Nothing
            Just stat ->
                Just stat.dots

isStat : String -> Stat -> Bool
isStat name stat =
    stat.name == name

changeStatInList : List Stat -> Stat -> List Stat
changeStatInList stats stat =
    List.map ( changeStatIfSame stat ) stats

changeStatIfSame : Stat -> Stat -> Stat
changeStatIfSame newStat oldStat =
    if isStat newStat.name oldStat
    then
        newStat
    else
        oldStat

changeDotsTo : Stat -> Int -> Stat
changeDotsTo stat value =
    { stat | dots = BoundedInt.changeTo stat.dots value }